import 'dart:math';

import 'package:flutter/material.dart';
import 'package:sheet/sheet.dart';

class AdvancedSnapSheetPageExample extends StatefulWidget {
  @override
  _AdvancedSnapSheetPageExampleState createState() => _AdvancedSnapSheetPageExampleState();
}

class _AdvancedSnapSheetPageExampleState extends State<AdvancedSnapSheetPageExample>
    with TickerProviderStateMixin {
  late SheetController controller;

  @override
  void initState() {
    controller = SheetController();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.grey[200],
      appBar: PreferredSize(
        preferredSize: Size.fromHeight(kToolbarHeight),
        child: AnimatedBuilder(
          animation: controller.animation,
          builder: (context, child) {
            final bool sheetBar = controller.animation.value == 1;
            return AnimatedTheme(
              data: !sheetBar
                  ? ThemeData(accentColor: Colors.grey[200])
                  : ThemeData.dark(),
              child: AppBar(
                automaticallyImplyLeading: false,
                title: !sheetBar ? Text('Map') : Text('Locations'),
              ),
            );
          },
        ),
      ),
      bottomNavigationBar: BottomNavigationBar(
        onTap: (index) {
          controller.relativeAnimateTo(0.1, duration: Duration(milliseconds: 200), curve: Curves.easeIn);
        },
        items: [
          BottomNavigationBarItem(label: 'Option 1', icon: Icon(Icons.location_city)),
          BottomNavigationBarItem(label: 'Option 2', icon: Icon(Icons.location_city)),
        ],
      ),
      body: LayoutBuilder(
        builder: (context, constraints) {
          return Stack(
            children: [
              Positioned.fill(
                child: Image.asset(
                  'assets/map.png',
                  fit: BoxFit.cover,
                ),
              ),
              AnimatedBuilder(
                  animation: controller.animation,
                  builder: (context, child) {
                    return Positioned(
                        right: 0,
                        bottom: constraints.biggest.height * min(0.3, controller.animation.value),
                        child: Container(
                          margin: EdgeInsets.all(20),
                          child: Column(
                            crossAxisAlignment: CrossAxisAlignment.end,
                            children: [
                              FloatingActionButton(
                                key: Key('value'),
                                heroTag: 'ggg',
                                onPressed: () {
                                  controller.relativeAnimateTo(
                                    0.1,
                                    duration: Duration(milliseconds: 200),
                                    curve: Curves.easeIn,
                                  );
                                },
                              ),
                              SizedBox(height: 8),
                              FloatingActionButton(
                                child: Icon(Icons.place),
                                onPressed: () {
                                  controller.relativeAnimateTo(
                                    0.1,
                                    duration: Duration(milliseconds: 200),
                                    curve: Curves.easeIn,
                                  );
                                },
                              ),
                            ],
                          ),
                        ));
                  }),
              Sheet(
                initialExtent: 120,
                controller: controller,
                physics: SnapSheetPhysics(
                  stops: [0, 0.1, 0.3, 0.5, 1],
                ),
                /*   snap: true,
                stops: [0, 0.1, 0.3, 0.5, 1], */
                child: Material(
                  elevation: 4,
                  color: Colors.white,
                  child: ListView(
                    shrinkWrap: true,
                    primary: true,
                    physics: BouncingScrollPhysics(),
                    children: ListTile.divideTiles(
                        context: context,
                        tiles: List.generate(100, (index) {
                          if (index == 0) {
                            return Container(
                              child: Text(
                                'Location',
                                style: Theme.of(context).textTheme.headline6,
                              ),
                              padding: EdgeInsets.all(20),
                              alignment: Alignment.center,
                            );
                          }
                          return ListTile(
                            title: Text('Item $index'),
                          );
                        })).toList(),
                  ),
                ),
              )
            ],
          );
        },
      ),
    );
  }

  @override
  void dispose() {
    controller.dispose();
    super.dispose();
  }
}
